"use strict";

import { Canvas, RoundedRect, Shadow } from '@shopify/react-native-skia';
import { createOuterShadowOffset, createStyles } from "../utils.js";
import { useMemo } from 'react';

/**
 * ShadowCanvas
 * -------------
 * Responsible for rendering the inset (or regular) shadow effect
 * inside a Skia <Canvas>. This includes the main shadow layer and
 * an optional reflected light layer. Typically used in conjunction
 * with a parent container like `ShadowView`.
 */
import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
export default function ShadowCanvas({
  width,
  height,
  style,
  shadowColor,
  shadowOffset,
  shadowBlur,
  backgroundColor,
  inset,
  reflectedLightColor,
  reflectedLightOffset,
  reflectedLightBlur,
  ...props
}) {
  /**
   * Determine if the reflected light layer should be rendered.
   * If `isReflectedLightEnabled` is explicitly set, use that.
   * Otherwise, default to rendering reflected light if `inset` is true.
   */
  const isReflectedLightEnabled = props.isReflectedLightEnabled !== undefined ? props.isReflectedLightEnabled : inset;

  /**
   * Extract a uniform corner radius from `style.borderRadius`.
   * If no radius is provided, default to 0 for sharp corners.
   * Note: more advanced logic may be needed to handle differing
   * corner radii on each corner.
   */
  const boxRadius = Number(style ? style.borderRadius : 0) || 0;
  const styles = createStyles({
    width,
    height
  });

  // Prepare the shadow offset and blur for the main shadow layer.
  // You can overwrite these values with `style` property.
  const outerShadowOffset = useMemo(() => createOuterShadowOffset({
    inset,
    shadowColor,
    shadowBlur,
    shadowOffset
  }), [inset, shadowColor, shadowBlur, shadowOffset]);
  return /*#__PURE__*/_jsx(Canvas, {
    style: [outerShadowOffset, style, styles.canvas],
    children: /*#__PURE__*/_jsxs(RoundedRect, {
      x: 0,
      y: 0,
      width: width,
      height: height,
      r: boxRadius,
      color: backgroundColor // The background fill of the rect
      ,
      children: [inset && /*#__PURE__*/_jsx(Shadow, {
        dx: shadowOffset.width,
        dy: shadowOffset.height,
        blur: shadowBlur,
        color: shadowColor,
        inner: true
      }), isReflectedLightEnabled && /*#__PURE__*/_jsx(Shadow, {
        dx: reflectedLightOffset.width,
        dy: reflectedLightOffset.height,
        blur: reflectedLightBlur,
        color: reflectedLightColor,
        inner: true
      })]
    })
  });
}
//# sourceMappingURL=ShadowCanvas.js.map